import { SocketClient } from '../socket/socket_client';
import { ElectrumBalances } from '../types';
interface PersistencePolicy {
    maxRetry: 10;
    callback: null;
}
export declare class ElectrumClient extends SocketClient {
    private persistencePolicy?;
    private timeLastCall;
    private keepAliveHandle?;
    constructor(host: string, port: number, protocol: string, options?: {});
    connect(clientName: string, electrumProtocolVersion: string, persistencePolicy?: PersistencePolicy): Promise<void>;
    request(method: string, params: any[]): Promise<any>;
    /**
     * Ping the server to ensure it is responding, and to keep the session alive.
     * The server may disconnect clients that have sent no requests for roughly 10
     * minutes. It sends a ping request every 2 minutes. If the request fails it
     * logs an error and closes the connection.
     */
    keepAlive(): Promise<void>;
    close(): Promise<void>;
    onClose(): void;
    server_version(client_name: string, protocol_version: string): Promise<any>;
    server_banner(): Promise<any>;
    server_ping(): Promise<any>;
    server_addPeer(features: any): Promise<any>;
    server_donation_address(): Promise<any>;
    server_features(): Promise<any>;
    server_peers_subscribe(): Promise<any>;
    blockchain_address_getProof(address: string): Promise<any>;
    blockchain_scripthash_getBalance(scripthash: string): Promise<ElectrumBalances>;
    blockchain_scripthash_getHistory(scripthash: string): Promise<any>;
    blockchain_scripthash_getMempool(scripthash: string): Promise<any>;
    blockchain_scripthash_listunspent(scripthash: string): Promise<any>;
    blockchain_scripthash_subscribe(scripthash: string): Promise<any>;
    blockchain_scripthash_unsubscribe(scripthash: string): Promise<any>;
    blockchain_block_header(height: number, cpHeight?: number): Promise<any>;
    blockchain_block_headers(startHeight: number, count: number, cpHeight?: number): Promise<any>;
    blockchainEstimatefee(number: number): Promise<any>;
    blockchain_headers_subscribe(): Promise<any>;
    blockchain_relayfee(): Promise<any>;
    blockchain_transaction_broadcast(rawtx: any): Promise<any>;
    blockchain_transaction_get(tx_hash: string, verbose: boolean): Promise<any>;
    blockchain_transaction_getMerkle(tx_hash: string, height: number): Promise<any>;
    mempool_getFeeHistogram(): Promise<any>;
    blockchain_utxo_getAddress(tx_hash: string, index: number): Promise<any>;
    blockchain_numblocks_subscribe(): Promise<any>;
    blockchain_block_getChunk(index: number): Promise<any>;
    blockchain_address_getBalance(address: string): Promise<any>;
    blockchain_address_getHistory(address: string): Promise<any>;
    blockchain_address_getMempool(address: string): Promise<any>;
    blockchain_address_listunspent(address: string): Promise<any>;
    blockchain_address_subscribe(address: string): Promise<any>;
}
export {};
