"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketClient = void 0;
const events_1 = require("events");
const util_1 = require("./util");
const socket_client_tcp_1 = require("./socket_client_tcp");
const socket_client_ws_1 = require("./socket_client_ws");
class SocketClient {
    constructor(host, port, protocol, options // TODO: determine type
    ) {
        this.host = host;
        this.port = port;
        this.protocol = protocol;
        this.options = options;
        this.id = 0;
        this.status = 0;
        this.callback_message_queue = [];
        this.subscribe = new events_1.EventEmitter();
        this.messageParser = new util_1.MessageParser((body, n) => {
            this.onMessage(body, n);
        });
        switch (protocol) {
            case 'tcp':
            case 'tls':
            case 'ssl':
                this.client = new socket_client_tcp_1.TCPSocketClient(this, host, port, protocol, options);
                break;
            case 'ws':
            case 'wss':
                this.client = new socket_client_ws_1.WebSocketClient(this, host, port, protocol, options);
                break;
            default:
                throw new Error(`invalid protocol: [${protocol}]`);
        }
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.status === 1) {
                return Promise.resolve();
            }
            this.status = 1;
            return this.client.connect();
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.status === 0) {
                return;
            }
            yield this.client.close();
            this.status = 0;
        });
    }
    response(msg) {
        const callback = this.callback_message_queue[msg.id];
        if (callback) {
            delete this.callback_message_queue[msg.id];
            if (msg.error) {
                callback(msg.error.message);
            }
            else {
                callback(null, msg.result);
            }
        }
        else {
            console.log("Can't get callback");
        }
    }
    onMessage(body, n) {
        const msg = JSON.parse(String(body));
        if (msg instanceof Array) {
            // don't support batch request
        }
        else {
            if (msg.id !== void 0) {
                this.response(msg);
            }
            else {
                this.subscribe.emit(msg.method, msg.params);
            }
        }
    }
    onConnect() { }
    onClose(event) {
        this.status = 0;
        this.callback_message_queue.forEach((value, index) => {
            this.callback_message_queue[index](new Error('close connect'));
            delete this.callback_message_queue[index];
        });
    }
    onRecv(chunk) {
        this.messageParser.run(chunk);
    }
    onEnd(error) {
        console.log(`onEnd: [${error}]`);
    }
    onError(error) {
        console.log(`onError: [${error}]`);
    }
}
exports.SocketClient = SocketClient;
