'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TCPSocketClient = void 0;
const tls_1 = __importDefault(require("tls"));
const net_1 = __importDefault(require("net"));
const TIMEOUT = 25000;
class TCPSocketClient {
    constructor(self, host, port, protocol, options) {
        this.self = self;
        this.host = host;
        this.port = port;
        this.protocol = protocol;
        this.options = options;
        let conn;
        switch (protocol) {
            case 'tcp':
                conn = new net_1.default.Socket();
                break;
            case 'tls':
            case 'ssl':
                try {
                    // @ts-ignore
                    conn = new tls_1.default.TLSSocket(undefined, options);
                }
                catch (e) {
                    console.log('connection error', e.message);
                    throw new Error(`Unable to connect: ${e.message}`);
                }
                break;
            default:
                throw new Error(`not supported protocol: ${protocol}`);
        }
        initialize(self, conn);
        this.client = conn;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            const client = this.client;
            return new Promise((resolve, reject) => {
                const errorHandler = (e) => reject(e);
                client.connect(this.port, this.host, () => {
                    client.removeListener('error', errorHandler);
                    resolve(undefined);
                });
                client.on('error', errorHandler);
            });
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            this.client.end();
            this.client.destroy();
        });
    }
    send(data) {
        this.client.write(data);
    }
}
exports.TCPSocketClient = TCPSocketClient;
function initialize(self, conn) {
    conn.setTimeout(TIMEOUT);
    conn.setEncoding('utf8');
    conn.setKeepAlive(true, 0);
    conn.setNoDelay(true);
    conn.on('connect', () => {
        conn.setTimeout(0);
        self.onConnect();
    });
    conn.on('close', e => {
        self.onClose(e);
    });
    conn.on('timeout', () => {
        const e = new Error('ETIMEDOUT');
        conn.emit('error', e);
    });
    conn.on('data', chunk => {
        conn.setTimeout(0);
        self.onRecv(chunk);
    });
    conn.on('end', (e) => {
        conn.setTimeout(0);
        self.onEnd(e);
    });
    conn.on('error', e => {
        self.onError(e);
    });
}
