"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketClient = void 0;
const websocket_1 = require("websocket");
class WebSocketClient {
    constructor(self, host, port, protocol, options) {
        this.self = self;
        this.host = host;
        this.port = port;
        this.protocol = protocol;
        this.options = options;
        const url = `${this.protocol}://${this.host}:${this.port}`;
        // https://github.com/theturtle32/WebSocket-Node/blob/master/docs/W3CWebSocket.md#constructor
        this.client = new websocket_1.w3cwebsocket(url, undefined, undefined, undefined, this.options);
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.client.onerror = (error) => {
                    this.self.onError(error);
                };
                this.client.onclose = event => {
                    this.self.onClose(event);
                    reject(new Error(`websocket connection closed: code: [${event.code}], reason: [${event.reason}]`));
                };
                this.client.onmessage = message => {
                    this.self.onMessage(message.data);
                };
                this.client.onopen = () => {
                    if (this.client.readyState === this.client.OPEN) {
                        this.self.onConnect();
                        resolve(undefined);
                    }
                };
            });
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            this.client.close(1000, 'close connection');
        });
    }
    send(data) {
        this.client.send(data);
    }
}
exports.WebSocketClient = WebSocketClient;
